'use client';
import React, { useState, useEffect, useCallback } from 'react';
import Image from 'next/image';
import {
  DrawerClose,
  DrawerContent,
  DrawerFooter,
  DrawerHeader,
  DrawerTitle,
  Pagination,
  PaginationContent,
  PaginationLink,
  PaginationItem,
  PaginationNext,
  PaginationPrevious,
  ScrollArea,
  Table,
  TableBody,
  TableCell,
  TableHead,
  TableHeader,
  TableRow,
  Button,
} from '@/components/ui';
import { getUserHistory, getAllHistory } from '@/lib/hooks';
import type { Profile } from '@/utils/supabase';
import type { UserWithStatus } from '@/lib/hooks';
import { toast } from 'sonner';

type HistoryDrawerProps = {
  user?: Profile | null;
  className?: string;
};

export const HistoryDrawer: React.FC<HistoryDrawerProps> = ({
  user = null,
  className = 'max-w-4xl mx-auto',
}) => {
  const [history, setHistory] = useState<UserWithStatus[]>([]);
  const [page, setPage] = useState<number>(1);
  const [totalPages, setTotalPages] = useState<number>(1);
  const [totalCount, setTotalCount] = useState<number>(0);
  const [loading, setLoading] = useState<boolean>(true);
  const perPage = 50;

  const fetchHistory = useCallback(
    async (currentPage: number) => {
      setLoading(true);
      try {
        let response;
        if (user && user.id !== '') {
          response = await getUserHistory(user.id, currentPage, perPage);
        } else {
          response = await getAllHistory(currentPage, perPage);
        }
        if (!response.success) {
          throw new Error(response.error);
        }
        setHistory(response.data.statuses);
        setTotalPages(response.data.meta.total_pages);
        setTotalCount(response.data.meta.total_count);
      } catch (error) {
        toast.error(`Error fetching history: ${error as Error}`);
        setHistory([]);
        setTotalPages(1);
        setTotalCount(0);
      } finally {
        setLoading(false);
      }
    },
    [user, perPage],
  );

  useEffect(() => {
    fetchHistory(page).catch((error) => {
      toast.error(`Error fetching history: ${error as Error}`);
    });
  }, [page, fetchHistory]);

  const handlePageChange = (newPage: number) => {
    if (newPage >= 1 && newPage <= totalPages) setPage(newPage);
  };

  const formatTime = (timestamp: string) => {
    const date = new Date(timestamp);
    return date.toLocaleString('en-US', {
      year: 'numeric',
      month: 'short',
      day: 'numeric',
      hour: 'numeric',
      minute: '2-digit',
      hour12: true,
    });
  };

  return (
    <DrawerContent className={className}>
      <DrawerHeader>
        <DrawerTitle>
          <div className='flex flex-row items-center justify-center py-4'>
            <Image
              src='/favicon.png'
              alt='Tech Tracker Logo'
              width={60}
              height={60}
              className='w-8 h-8 md:w-12 md:h-12'
            />
            <h1 className='text-lg md:text-2xl lg:text-4xl font-bold pl-2 md:pl-4'>
              {user && user.id !== ''
                ? `${user.full_name}'s History`
                : 'All History'}
            </h1>
          </div>
          {totalCount > 0 && (
            <p className='text-sm text-muted-foreground text-center'>
              {totalCount} total entries
            </p>
          )}
        </DrawerTitle>
      </DrawerHeader>
      <div className='px-4'>
        <ScrollArea className='h-96 w-full'>
          {loading ? (
            <div className='flex justify-center items-center h-32'>
              <div className='animate-spin rounded-full h-8 w-8 border-b-2 border-primary'></div>
            </div>
          ) : history.length === 0 ? (
            <div className='flex justify-center items-center h-32'>
              <p className='text-muted-foreground'>No history found</p>
            </div>
          ) : (
            <Table>
              <TableHeader>
                <TableRow>
                  <TableHead className='font-semibold'>Name</TableHead>
                  <TableHead className='font-semibold'>Status</TableHead>
                  <TableHead className='font-semibold'>Updated By</TableHead>
                  <TableHead className='font-semibold text-right'>
                    Date & Time
                  </TableHead>
                </TableRow>
              </TableHeader>
              <TableBody>
                {history.map((entry) => (
                  <TableRow key={entry.id}>
                    <TableCell className='font-medium'>
                      {entry.user.full_name}
                    </TableCell>
                    <TableCell className='max-w-xs'>
                      <div className='truncate' title={entry.status}>
                        {entry.status}
                      </div>
                    </TableCell>
                    <TableCell className='text-sm text-muted-foreground'>
                      {entry.updated_by?.full_name ?? ''}
                    </TableCell>
                    <TableCell className='text-right text-sm'>
                      {formatTime(entry.created_at)}
                    </TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          )}
        </ScrollArea>
      </div>
      <DrawerFooter>
        {totalPages > 1 && (
          <Pagination>
            <PaginationContent>
              {page > 1 && (
                <PaginationItem>
                  <PaginationPrevious
                    href='#'
                    onClick={(e) => {
                      e.preventDefault();
                      handlePageChange(page - 1);
                    }}
                  />
                </PaginationItem>
              )}

              {totalPages > 10 ? (
                <div className='flex items-center gap-2 text-sm text-muted-foreground'>
                  <span>Page</span>
                  <span className='font-bold text-foreground'>{page}</span>
                  <span>of</span>
                  <span className='font-semibold text-foreground'>
                    {totalPages}
                  </span>
                </div>
              ) : (
                Array.from({ length: totalPages }).map((_, idx) => (
                  <PaginationItem key={idx}>
                    <PaginationLink
                      href='#'
                      isActive={page === idx + 1}
                      onClick={(e) => {
                        e.preventDefault();
                        handlePageChange(idx + 1);
                      }}
                    >
                      {idx + 1}
                    </PaginationLink>
                  </PaginationItem>
                ))
              )}

              {page < totalPages && (
                <PaginationItem>
                  <PaginationNext
                    href='#'
                    onClick={(e) => {
                      e.preventDefault();
                      handlePageChange(page + 1);
                    }}
                  />
                </PaginationItem>
              )}
            </PaginationContent>
          </Pagination>
        )}

        <DrawerClose asChild>
          <Button variant='outline' className='mt-4'>
            Close
          </Button>
        </DrawerClose>
      </DrawerFooter>
    </DrawerContent>
  );
};
