'use client';
import { useState, useEffect } from 'react';
import type React from 'react';
import { useAuth, useTVMode } from '@/components/context';
import type { UserWithStatus } from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { StatusMessage, SubmitButton } from '@/components/default';
import { ConnectionStatus, HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Card, CardContent } from '@/components/ui/card';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Calendar, CheckCircle2 } from 'lucide-react';
import { useStatusData, useStatusSubscription } from '@/lib/hooks';
import { formatTime, formatDate } from '@/lib/utils';
import Link from 'next/link';

type ListProps = {
  initialStatuses: UserWithStatus[];
};

export const StatusList = ({ initialStatuses = [] }: ListProps) => {
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);
  const [updateStatusMessage, setUpdateStatusMessage] = useState('');

  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    newStatuses,
    updateStatusMutation,
  } = useStatusData({
    initialData: initialStatuses,
    enabled: isAuthenticated,
  });

  const { connectionStatus, connect: reconnect } = useStatusSubscription(() => {
    refetch().catch((error) => {
      console.error('Error refetching statuses:', error);
    });
  });

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      setUpdateStatusMessage(
        'Error: You must be signed in to update technician statuses!',
      );
      return;
    }

    if (statusInput.length < 3 || statusInput.length > 80) {
      setUpdateStatusMessage(
        'Error: Your status must be between 3 & 80 characters long!',
      );
      return;
    }

    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });

    setSelectedUsers([]);
    setStatusInput('');
    setUpdateStatusMessage('');
  };

  const handleCardSelect = (user: UserWithStatus, e: React.MouseEvent) => {
    // Prevent selection if clicking on profile elements
    if ((e.target as HTMLElement).closest('[data-profile-trigger]')) {
      return;
    }

    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user],
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0,
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName:
      'flex flex-col mx-auto items-center\
      sm:w-5/6 md:w-3/4 lg:w-2/3 xl:w-1/2 min-w-[450px]',
    on: 'mt-8',
    off: 'px-10',
  });

  const headerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'w-full',
    on: 'hidden',
    off: 'flex mb-3 justify-between items-center',
  });

  const cardContainerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'w-full space-y-2',
  });

  return (
    <div className={containerClassName}>
      <div className={headerClassName}>
        <div className='flex items-center gap-4'>
          <Button
            variant='outline'
            size='sm'
            onClick={handleSelectAllChange}
            className='flex items-center gap-2'
          >
            <CheckCircle2
              className={`w-4 h-4 ${selectAll ? 'text-primary' : ''}`}
            />
            {selectAll ? 'Deselect All' : 'Select All'}
          </Button>
          {!tvMode && (
            <div className='flex items-center gap-2 text-xs'>
              <span className='text-muted-foreground'>Miss the old table?</span>
              <Link
                href='/status/table'
                className='font-medium hover:underline'
              >
                Find it here!
              </Link>
            </div>
          )}
        </div>
        <div className='flex items-center gap-2'>
          <ConnectionStatus
            status={connectionStatus}
            onReconnect={reconnect}
            showAsButton={connectionStatus === 'disconnected'}
          />
        </div>
      </div>

      <div className={cardContainerClassName}>
        {usersWithStatuses.map((userWithStatus) => {
          const isSelected = selectedUsers.some(
            (u) => u.user.id === userWithStatus.user.id,
          );
          const isNewStatus = newStatuses.has(userWithStatus);
          const isUpdatedByOther =
            userWithStatus.updated_by &&
            userWithStatus.updated_by.id !== userWithStatus.user.id;

          return (
            <Card
              key={userWithStatus.user.id}
              className={`
                relative transition-all duration-200 cursor-pointer hover:shadow-md
                ${tvMode ? 'p-4' : 'p-3'}
                ${isSelected ? 'ring-2 ring-primary bg-primary/5 shadow-md' : 'hover:bg-muted/30'}
                ${isNewStatus ? 'animate-in slide-in-from-top-2 duration-500 bg-green-50 border-green-200' : ''}
              `}
              onClick={(e) => handleCardSelect(userWithStatus, e)}
            >
              {isSelected && (
                <div className='absolute top-2 right-2 text-primary'>
                  <CheckCircle2
                    className={`${tvMode ? 'w-6 h-6' : 'w-5 h-5'}`}
                  />
                </div>
              )}

              <CardContent className='p-0'>
                <div className='flex items-start gap-3'>
                  {/* Profile Section - Clickable for history */}
                  <Drawer>
                    <DrawerTrigger asChild>
                      <div
                        data-profile-trigger
                        className='flex-shrink-0 cursor-pointer hover:opacity-80 transition-opacity'
                        onClick={() =>
                          setSelectedHistoryUser(userWithStatus.user)
                        }
                      >
                        <BasedAvatar
                          src={userWithStatus.user.avatar_url}
                          fullName={userWithStatus.user.full_name}
                          className={tvMode ? 'w-16 h-16' : 'w-12 h-12'}
                        />
                      </div>
                    </DrawerTrigger>
                    {selectedHistoryUser === userWithStatus.user && (
                      <HistoryDrawer user={selectedHistoryUser} />
                    )}
                  </Drawer>

                  {/* Content Section */}
                  <div className='flex-1'>
                    {/* Header with name and timestamp */}
                    <div className='flex items-start justify-between mb-2'>
                      <div>
                        <Drawer>
                          <DrawerTrigger asChild>
                            <h3
                              data-profile-trigger
                              className={`
                                font-semibold cursor-pointer hover:text-primary/80 truncate
                                ${tvMode ? 'text-3xl' : 'text-2xl'}
                              `}
                              onClick={() =>
                                setSelectedHistoryUser(userWithStatus.user)
                              }
                            >
                              {userWithStatus.user.full_name}
                            </h3>
                          </DrawerTrigger>
                          {selectedHistoryUser === userWithStatus.user && (
                            <HistoryDrawer user={selectedHistoryUser} />
                          )}
                        </Drawer>
                        <div
                          className={`pl-2 pr-15 pt-2 ${tvMode ? 'text-2xl' : 'text-xl'}`}
                        >
                          <p>{userWithStatus.status}</p>
                        </div>
                      </div>
                      <div className='flex flex-col items-end px-2 gap-2 text-muted-foreground flex-shrink-0'>
                        <div className='flex items-center gap-2 flex-shrink-0 w-full'>
                          <Clock
                            className={`${tvMode ? 'w-6 h-6' : 'w-5 h-5'}`}
                          />
                          <span
                            className={`${tvMode ? 'text-2xl' : 'text-xl'}`}
                          >
                            {formatTime(userWithStatus.created_at)}
                          </span>
                        </div>
                        <div className='flex items-center gap-2 flex-shrink-0 w-full'>
                          <Calendar
                            className={`${tvMode ? 'w-6 h-6' : 'w-5 h-5'}`}
                          />
                          <span
                            className={`${tvMode ? 'text-2xl' : 'text-xl'}`}
                          >
                            {formatDate(userWithStatus.created_at)}
                          </span>
                        </div>
                        <div className='flex items-center gap-2 flex-shrink-0'>
                          {isUpdatedByOther && (
                            <div className='flex items-center gap-2'>
                              <BasedAvatar
                                src={userWithStatus.updated_by?.avatar_url}
                                fullName={userWithStatus.updated_by?.full_name}
                                className={`${tvMode ? 'w-6 h-6' : 'w-5 h-5'}`}
                              />
                              <span
                                className={`${tvMode ? 'text-base' : 'text-sm'}`}
                              >
                                <div className='flex flex-col'>
                                  <p>Updated by</p>
                                  {userWithStatus.updated_by?.full_name}
                                </div>
                              </span>
                            </div>
                          )}
                        </div>
                      </div>
                    </div>
                  </div>
                </div>
              </CardContent>
            </Card>
          );
        })}
      </div>

      {usersWithStatuses.length === 0 && (
        <Card className='p-8 text-center'>
          <p
            className={`text-muted-foreground ${tvMode ? 'text-2xl' : 'text-lg'}`}
          >
            No status updates have been made in the past day.
          </p>
        </Card>
      )}

      {!tvMode && (
        <Card className='p-6 mt-4 w-full'>
          <div className='flex flex-col gap-4'>
            <h3 className='text-lg font-semibold'>Update Status</h3>
            <div className='flex flex-col gap-4'>
              <div className='flex gap-4'>
                <Input
                  autoFocus
                  type='text'
                  placeholder='Enter status update...'
                  className='flex-1 text-2xl'
                  value={statusInput}
                  disabled={updateStatusMutation.isPending}
                  onChange={(e) => setStatusInput(e.target.value)}
                  onKeyDown={(e) => {
                    if (
                      e.key === 'Enter' &&
                      !e.shiftKey &&
                      !updateStatusMutation.isPending
                    ) {
                      e.preventDefault();
                      handleUpdateStatus();
                    }
                  }}
                />
                <SubmitButton
                  onClick={handleUpdateStatus}
                  disabled={updateStatusMutation.isPending}
                  className='px-6'
                >
                  {selectedUsers.length > 0
                    ? `Update ${selectedUsers.length} ${selectedUsers.length > 1 ? 'users' : 'user'}`
                    : 'Update Status'}
                </SubmitButton>
              </div>
              {updateStatusMessage &&
                (updateStatusMessage.includes('Error') ||
                updateStatusMessage.includes('error') ||
                updateStatusMessage.includes('failed') ||
                updateStatusMessage.includes('invalid') ? (
                  <StatusMessage message={{ error: updateStatusMessage }} />
                ) : (
                  <StatusMessage message={{ message: updateStatusMessage }} />
                ))}
            </div>
            <div className='flex justify-center mt-2'>
              <Drawer>
                <DrawerTrigger asChild>
                  <Button
                    variant='outline'
                    className={tvMode ? 'text-xl p-6' : ''}
                  >
                    View All Status History
                  </Button>
                </DrawerTrigger>
                <HistoryDrawer />
              </Drawer>
            </div>
          </div>
        </Card>
      )}
    </div>
  );
};
