'use client';
import { useState, useEffect } from 'react';
import { useAuth, useTVMode } from '@/components/context';
import type { UserWithStatus } from '@/lib/hooks';
import { BasedAvatar, Drawer, DrawerTrigger, Loading } from '@/components/ui';
import { StatusMessage, SubmitButton } from '@/components/default';
import { ConnectionStatus, HistoryDrawer } from '@/components/status';
import type { Profile } from '@/utils/supabase';
import { makeConditionalClassName } from '@/lib/utils';
import { Input } from '@/components/ui/input';
import { Button } from '@/components/ui/button';
import { RefreshCw, Clock, Calendar } from 'lucide-react';
import { useStatusSubscription, useStatusData } from '@/lib/hooks';
import { formatTime, formatDate } from '@/lib/utils';
import Link from 'next/link';

type TableProps = {
  initialStatuses: UserWithStatus[];
};

export const TechTable = ({ initialStatuses = [] }: TableProps) => {
  const { isAuthenticated } = useAuth();
  const { tvMode } = useTVMode();

  const [selectedUsers, setSelectedUsers] = useState<UserWithStatus[]>([]);
  const [selectAll, setSelectAll] = useState(false);
  const [statusInput, setStatusInput] = useState('');
  const [selectedHistoryUser, setSelectedHistoryUser] =
    useState<Profile | null>(null);
  const [updateStatusMessage, setUpdateStatusMessage] = useState('');

  const {
    data: usersWithStatuses = initialStatuses,
    isLoading: loading,
    error,
    refetch,
    newStatuses,
    updateStatusMutation,
  } = useStatusData({
    initialData: initialStatuses,
    enabled: isAuthenticated,
  });
  // In your StatusList component
  const { connectionStatus, connect: reconnect } = useStatusSubscription(() => {
    refetch().catch((error) => {
      console.error('Error refetching statuses:', error);
    });
  });
  //const { connectionStatus, connect: reconnect } = useStatusSubscription({
  //enabled: isAuthenticated,
  //onStatusUpdate: () => {
  //refetch().catch((error) => {
  //console.error('Error refetching statuses:', error);
  //});
  //},
  //});

  const handleUpdateStatus = () => {
    if (!isAuthenticated) {
      setUpdateStatusMessage(
        'Error: You must be signed in to update technician statuses!',
      );
      return;
    }

    if (statusInput.length < 3 || statusInput.length > 80) {
      setUpdateStatusMessage(
        'Error: Your status must be between 3 & 80 characters long!',
      );
      return;
    }

    updateStatusMutation.mutate({
      usersWithStatuses: selectedUsers,
      status: statusInput.trim(),
    });

    setSelectedUsers([]);
    setStatusInput('');
    setUpdateStatusMessage('');
  };

  const handleCheckboxChange = (user: UserWithStatus) => {
    setSelectedUsers((prev) =>
      prev.some((u) => u.user.id === user.user.id)
        ? prev.filter((prevUser) => prevUser.user.id !== user.user.id)
        : [...prev, user],
    );
  };

  const handleSelectAllChange = () => {
    if (selectAll) {
      setSelectedUsers([]);
    } else {
      setSelectedUsers(usersWithStatuses);
    }
    setSelectAll(!selectAll);
  };

  useEffect(() => {
    setSelectAll(
      selectedUsers.length === usersWithStatuses.length &&
        usersWithStatuses.length > 0,
    );
  }, [selectedUsers.length, usersWithStatuses.length]);

  if (loading) {
    return (
      <div className='flex justify-center items-center min-h-[400px]'>
        <Loading className='w-full' alpha={0.5} />
      </div>
    );
  }

  if (error) {
    return (
      <div className='flex flex-col justify-center items-center min-h-[400px] gap-4'>
        <p className='text-red-500'>Error loading status updates</p>
        <Button onClick={() => refetch()} variant='outline'>
          <RefreshCw className='w-4 h-4 mr-2' />
          Retry
        </Button>
      </div>
    );
  }

  const containerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'mx-auto',
    on: 'lg:w-11/12 w-full',
    off: 'w-5/6',
  });
  const headerClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'w-full mb-2 flex justify-between',
    on: 'mt-25',
    off: 'mb-2',
  });
  const thClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'py-4 px-4 border font-semibold ',
    on: 'lg:text-5xl xl:min-w-[420px]',
    off: 'lg:text-4xl xl:min-w-[320px]',
  });
  const tdClassName = makeConditionalClassName({
    context: tvMode,
    defaultClassName: 'py-2 px-2 border',
    on: 'lg:text-4xl',
    off: 'lg:text-3xl',
  });
  const tCheckboxClassName = `py-3 px-4 border`;
  const checkBoxClassName = `lg:scale-200 cursor-pointer`;

  return (
    <div className={containerClassName}>
      <div className={headerClassName}>
        <div className='flex items-center gap-2'>
          <ConnectionStatus
            status={connectionStatus}
            onReconnect={reconnect}
            showAsButton={connectionStatus === 'disconnected'}
          />
          {!tvMode && (
            <div className='flex flex-row gap-2 text-xs'>
              <p className='text-muted-foreground'>Tired of the old table? </p>
              <Link
                href='/status/list'
                className='italic font-semibold hover:text-primary/80'
              >
                Try the new status list!
              </Link>
            </div>
          )}
        </div>
      </div>

      <table className='w-full text-center rounded-md'>
        <thead>
          <tr className='bg-muted'>
            {!tvMode && (
              <th className={tCheckboxClassName}>
                <input
                  type='checkbox'
                  className={checkBoxClassName}
                  checked={selectAll}
                  onChange={handleSelectAllChange}
                />
              </th>
            )}
            <th className={thClassName}>Technician</th>
            <th className={thClassName}>
              <Drawer>
                <DrawerTrigger className='hover:text-foreground/60 cursor-pointer'>
                  Status
                </DrawerTrigger>
                <HistoryDrawer />
              </Drawer>
            </th>
            <th className={thClassName}>Updated At</th>
          </tr>
        </thead>
        <tbody>
          {usersWithStatuses.map((userWithStatus, index) => {
            const isSelected = selectedUsers.some(
              (u) => u.user.id === userWithStatus.user.id,
            );
            const isNewStatus = newStatuses.has(userWithStatus);

            return (
              <tr
                key={userWithStatus.user.id}
                className={`
                  ${index % 2 === 0 ? 'bg-muted/50' : 'bg-background'}
                  ${isSelected ? 'ring-2 ring-primary' : ''}
                  ${isNewStatus ? 'animate-pulse bg-primary/5 border-primary/20' : ''}
                  hover:bg-muted/75 transition-all duration-300
                `}
              >
                {!tvMode && (
                  <td className={tCheckboxClassName}>
                    <input
                      type='checkbox'
                      className={checkBoxClassName}
                      checked={isSelected}
                      onChange={() => handleCheckboxChange(userWithStatus)}
                    />
                  </td>
                )}
                <td className={tdClassName}>
                  <div className='flex items-center gap-3'>
                    <BasedAvatar
                      src={userWithStatus.user.avatar_url}
                      fullName={userWithStatus.user.full_name}
                      className={tvMode ? 'w-16 h-16' : 'w-12 h-12'}
                    />
                    <div>
                      <p
                        className={`font-semibold ${tvMode ? 'text-5xl' : 'text-4xl'}`}
                      >
                        {userWithStatus.user.full_name ?? 'Unknown User'}
                      </p>
                      {userWithStatus.updated_by &&
                        userWithStatus.updated_by.id !==
                          userWithStatus.user.id && (
                          <div className='flex items-center gap-1 text-muted-foreground'>
                            <BasedAvatar
                              src={userWithStatus.updated_by?.avatar_url}
                              fullName={userWithStatus.updated_by?.full_name}
                              className='w-5 h-5'
                            />
                            <span className={tvMode ? 'text-xl' : 'text-base'}>
                              Updated by {userWithStatus.updated_by.full_name}
                            </span>
                          </div>
                        )}
                    </div>
                  </div>
                </td>
                <td className={tdClassName}>
                  <Drawer>
                    <DrawerTrigger
                      className='text-center w-full p-2 rounded-md hover:bg-muted transition-colors'
                      onClick={() =>
                        setSelectedHistoryUser(userWithStatus.user)
                      }
                    >
                      {userWithStatus.status}
                    </DrawerTrigger>
                    {selectedHistoryUser === userWithStatus.user && (
                      <HistoryDrawer user={selectedHistoryUser} />
                    )}
                  </Drawer>
                </td>
                <td className={tdClassName}>
                  <div className='flex w-full'>
                    <div className='flex items-start xl:w-1/6'></div>
                    <div className='flex flex-col my-auto items-start'>
                      <div className='flex gap-4 my-1'>
                        <Clock
                          className={`${tvMode ? 'lg:w-11 lg:h-11' : 'lg:w-9 lg:h-9'}`}
                        />
                        {formatTime(userWithStatus.created_at)}
                      </div>
                      <div className='flex gap-4 my-1'>
                        <Calendar
                          className={`${tvMode ? 'lg:w-11 lg:h-11' : 'lg:w-9 lg:h-9'}`}
                        />
                        {formatDate(userWithStatus.created_at)}
                      </div>
                    </div>
                  </div>
                </td>
              </tr>
            );
          })}
        </tbody>
      </table>

      {usersWithStatuses.length === 0 && (
        <div className='p-8 text-center'>
          <p
            className={`text-muted-foreground ${tvMode ? 'text-4xl' : 'text-lg'}`}
          >
            No status updates yet
          </p>
        </div>
      )}

      {updateStatusMessage &&
        (updateStatusMessage.includes('Error') ||
        updateStatusMessage.includes('error') ||
        updateStatusMessage.includes('failed') ||
        updateStatusMessage.includes('invalid') ? (
          <StatusMessage message={{ error: updateStatusMessage }} />
        ) : (
          <StatusMessage message={{ message: updateStatusMessage }} />
        ))}

      {!tvMode && (
        <div className='mx-auto flex flex-row items-center justify-center py-5 gap-4'>
          <Input
            autoFocus
            type='text'
            placeholder='New Status'
            className={
              'min-w-[120px] lg:max-w-[400px] py-6 px-3 rounded-xl \
                border bg-background lg:text-2xl focus:outline-none \
                focus:ring-2 focus:ring-primary'
            }
            value={statusInput}
            onChange={(e) => setStatusInput(e.target.value)}
            onKeyDown={(e) => {
              if (e.key === 'Enter' && !updateStatusMutation.isPending) {
                e.preventDefault();
                handleUpdateStatus();
              }
            }}
            disabled={updateStatusMutation.isPending}
          />
          <SubmitButton
            size='xl'
            className={
              'px-8 rounded-xl font-semibold lg:text-2xl \
              disabled:opacity-50 disabled:cursor-not-allowed \
              cursor-pointer'
            }
            onClick={handleUpdateStatus}
            disabled={updateStatusMutation.isPending}
            pendingText='Updating...'
          >
            {selectedUsers.length > 0
              ? `Update status for ${selectedUsers.length}
                ${selectedUsers.length > 1 ? 'users' : 'user'}`
              : 'Update status'}
          </SubmitButton>
        </div>
      )}

      {/* Global Status History Drawer */}
      {!tvMode && (
        <div className='flex justify-center mt-6'>
          <Drawer>
            <DrawerTrigger asChild>
              <Button
                variant='outline'
                className={tvMode ? 'text-3xl p-6' : ''}
              >
                View All Status History
              </Button>
            </DrawerTrigger>
            <HistoryDrawer />
          </Drawer>
        </div>
      )}
    </div>
  );
};
