'use client';
import { useState } from 'react';
import { useQuery, useMutation, useQueryClient } from '@tanstack/react-query';
import {
  getRecentUsersWithStatuses,
  updateStatuses,
  updateUserStatus,
  type UserWithStatus,
} from '@/lib/hooks';
import { QueryErrorCodes } from '@/components/context';
import { toast } from 'sonner';

type UseStatusDataOptions = {
  initialData?: UserWithStatus[];
  enabled?: boolean;
};

export const useStatusData = ({
  initialData = [],
  enabled = true,
}: UseStatusDataOptions = {}) => {
  const queryClient = useQueryClient();
  const [newStatuses, setNewStatuses] = useState<Set<UserWithStatus>>(
    new Set(),
  );

  const query = useQuery({
    queryKey: ['users-with-statuses'],
    queryFn: async () => {
      try {
        const response = await getRecentUsersWithStatuses();
        if (!response.success) throw new Error(response.error);
        return response.data;
      } catch (error) {
        toast.error(`Error fetching technicians: ${error as Error}`);
        throw error;
      }
    },
    enabled,
    refetchInterval: 30000, // 30 seconds
    refetchOnWindowFocus: true,
    refetchOnMount: true,
    initialData,
    meta: { errCode: QueryErrorCodes.USERS_FETCH_FAILED },
  });

  const updateStatusMutation = useMutation({
    mutationFn: async ({
      usersWithStatuses,
      status,
    }: {
      usersWithStatuses: UserWithStatus[];
      status: string;
    }) => {
      try {
        if (usersWithStatuses.length <= 0) {
          const result = await updateUserStatus(status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        } else {
          const result = await updateStatuses(usersWithStatuses, status);
          if (!result.success) throw new Error(result.error);
          return result.data;
        }
      } catch (error) {
        console.error(`Error updating statuses: ${error as Error}`);
        throw error;
      }
    },
    meta: { errCode: QueryErrorCodes.STATUS_UPDATE_FAILED },
    onMutate: async ({ usersWithStatuses, status }) => {
      await queryClient.cancelQueries({ queryKey: ['users-with-statuses'] });
      const previousData = queryClient.getQueryData<UserWithStatus[]>([
        'users-with-statuses',
      ]);

      if (previousData && usersWithStatuses.length > 0) {
        const now = new Date().toISOString();
        const optimisticData = previousData.map((userStatus) => {
          if (
            usersWithStatuses.some(
              (selected) => selected.user.id === userStatus.user.id,
            )
          ) {
            return { ...userStatus, status, created_at: now };
          }
          return userStatus;
        });
        queryClient.setQueryData(['users-with-statuses'], optimisticData);

        // Add animation to optimistically updated statuses
        setNewStatuses((prev) => new Set([...prev, ...usersWithStatuses]));
        setTimeout(() => {
          setNewStatuses((prev) => {
            const updated = new Set(prev);
            usersWithStatuses.forEach((updatedStatus) =>
              updated.delete(updatedStatus),
            );
            return updated;
          });
        }, 1000);
      }

      return { previousData };
    },
    onSuccess: (data) => {
      queryClient
        .invalidateQueries({ queryKey: ['users-with-statuses'] })
        .catch((error) => console.error(`Error invalidating query: ${error}`));

      if (!data) return;

      data.forEach((statusUpdate) => {
        toast.success(
          `${statusUpdate.user.full_name}'s status updated to '${statusUpdate.status}'.`,
        );
      });
    },
    onError: (error, _variables, context) => {
      if (context?.previousData) {
        queryClient.setQueryData(['users-with-statuses'], context.previousData);
      }
      toast.error(`Error updating statuses: ${error}`);
    },
  });

  return {
    ...query,
    newStatuses,
    updateStatusMutation,
  };
};
